import subprocess
import re

list_prefix = "gsettings list-keys "
check_version = list_prefix + "org.gnome.settings-daemon.plugins.media-keys"
new_task_action = "gtg_new_task"
new_task_name = "GTG New Task"
get_temp = "gsettings get "
path = "org.gnome.settings-daemon.plugins.media-keys "
gsettings_get_list = get_temp + path + "custom-keybindings"
set_temp = "gsettings set "
gsettings_set_list = set_temp + path + "custom-keybindings"
key_path1 = "org.gnome.settings-daemon.plugins.media-keys.custom-keybinding:"
path2 = "/org/gnome/settings-daemon/plugins/media-keys/custom-keybindings/"
gsettings_get = get_temp + key_path1 + path2 + "custom"
gsettings_set = set_temp + key_path1 + path2 + "custom"
gconf_get = "gconftool-2 --get /desktop/gnome/keybindings/custom"
gconf_set = "gconftool-2 --type string --set /desktop/gnome/keybindings/custom"


def get_saved_binding():
    """ Get the current shortcut if the task exists """
    list_keys = call_subprocess(check_version, None, None, None)
    list_keys = list_keys.split("\n")
    if "custom-keybindings" in list_keys:
        binding = get_shortcut_from_dconf()
    else:
        binding = get_shortcut_from_gconf()
    return binding


def get_shortcut_from_dconf():
    """ If system uses dconf, then get the shortcut via gsettings """
    dconf_out = call_subprocess(gsettings_get_list, None, None, None)
    custom_shortcuts_list = re.findall(r'custom[0-9]+', dconf_out)

    for i in custom_shortcuts_list:
        to_access = i[len(i)-1]
        get_cmd = call_subprocess(gsettings_get, None, to_access, "/ command")

        if re.findall(new_task_action, get_cmd):
            bind = "/ binding"
            get_bind = call_subprocess(gsettings_get, None, to_access, bind)
            if get_bind.endswith('\n'):
                get_bind = get_bind[:len(get_bind)-1]
            return get_bind
    return None


def get_shortcut_from_gconf():
    """ If system uses gconf, then get the shortcut via gconftool-2 """
    i=0
    while(1):
        get_action = call_subprocess(gconf_get, None, i, "/action")

        if get_action == (new_task_action + "\n"):
            get_bind = call_subprocess(gconf_get, None, i, "/binding")
            if get_bind.endswith('\n'):
                get_bind = get_bind[:len(get_bind)-1]
            return get_bind

        elif get_action == "":
            get_name = call_subprocess(gconf_get, None, i, "/name")
            if get_name == "":
                return None
    i += 1


def on_shortcut_change(binding, button_state):
    """ When user has entered a new shortcut """
    if button_state == True:
        list_keys = call_subprocess(check_version, None, None, None)
        list_keys = list_keys.split("\n")
        if "custom-keybindings" in list_keys:
            add_shortcut_to_dconf(binding)
        else:
            add_shortcut_to_gconf(binding)


def add_shortcut_to_dconf(binding):
    """ If system uses dconf, then set the new shortcut via gsettings """
    dconf_out = call_subprocess(gsettings_get_list, None, None, None)
    custom_keys = re.findall(r'custom[0-9]+', dconf_out)

    for i in custom_keys:
        to_access = i[len(i)-1]
        get_cmd = call_subprocess(gsettings_get, None, to_access, "/ command")

        if re.findall(new_task_action, get_cmd):
            call_subprocess(gsettings_set, binding, to_access, "/ binding")
            return

    a=[]
    for item in custom_keys:
        a.append(int(re.findall(r'[0-9]+', item)[0]))
    if a == []:
        index = 0
    else:
        a.sort()
        index = a[len(a)-1] + 1
    prefix = "['/org/gnome/settings-daemon/plugins/media-keys/"
    prefix = prefix + "custom-keybindings/custom"
    append_this = prefix + str(index) + "/']"
    call_subprocess(gsettings_set, new_task_action, index, "/ command")
    call_subprocess(gsettings_set, binding, index, "/ binding")
    call_subprocess(gsettings_set, new_task_name, index, "/ name")

    if index == 0:
        call_subprocess(gsettings_set_list, None, None, ' ' + append_this)

    else:
        result_list = dconf_out[:len(dconf_out)-2] + ", "
        result_list = result_list + append_this[1:len(append_this)-1] + "]"
        call_subprocess(gsettings_set_list, result_list, None, None)


def add_shortcut_to_gconf(binding):
    """ If system uses gconf, then set the new shortcut via gconftool-2 """
    i=0
    while(1):
        get_action = call_subprocess(gconf_get, None, i, "/action")

        if get_action == (new_task_action + "\n"):
            binding_ans = call_subprocess(gconf_set, binding, i, "/binding")
            break

        if get_action == "":
            action = call_subprocess(gconf_set, new_task_action, i, "/action")
            binding = call_subprocess(gconf_set, binding, i, "/binding")
            name = call_subprocess(gconf_set, new_task_name, i, "/name")
            break
    i += 1


def call_subprocess(cmd, append_this, i, key):
    """ Sets the values in either dconf or gconf """
    if i != None:
        cmd = cmd + str(i)
    if key != None:
        cmd = cmd + key
    cmd = cmd.split(" ")
    if append_this != None:
        cmd.append(append_this)
    out = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    return out.communicate()[0]


def check_invalidity(binding, key, mods):
    """ Checks if the user has entered inappropriate shortcut """
    if mods == 0:
        if (key >= 97 and key <= 122):
            # key is an alphabet
            return 1
        elif (key >= 48 and key <= 57):
            # key is a number
            return 1
        elif (key >= 65361 and key <= 65364):
            # key is one of the arrow keys
            return 1
        elif key == 32:
            # key is 'space'
            return 1
    return 0
    
#print "At startup, binding = " + str(get_saved_binding())
#on_shortcut_change("<Shift><Alt>y", True)
